@@ -1,18 +1,17 @@
 from datetime import datetime
+from functools import cached_property
 from json import dumps
 from json import loads
 from os import makedirs
+from os.path import abspath
 from os.path import dirname
 from os.path import join
 from re import sub
 from sqlite3 import Connection
 from sqlite3 import Cursor
 from sqlite3 import connect
-from typing import Dict
 from typing import Generator
-from typing import List
 from typing import Optional
-from typing import Tuple
 from typing import Union
 
 from chardet import detect as detect_encoding
@@ -33,12 +32,12 @@ from .update import update_database
 
 Key = Union[str, int, float]
 Value = Union[str, int, float, None]
-Entry = Dict[str, Union[List[str], Value]]
+Entry = dict[str, Union[list[str], Value]]
 
 
-def guess_extension(file: bytes, default: str = "") -> str:
+def guess_extension(file: Optional[bytes], default: str = "") -> str:
     if not file:
-        return ""
+        return default
     elif (file_type := filetype_guess_extension(file)) is None:
         return default if default else "txt" * (detect_encoding(file[:2048]).get("encoding", None) is not None)
     elif (ext := str(file_type)) in (exts := ("zip", "octet-stream")):
@@ -64,10 +63,7 @@ class FADatabaseTable:
     def __init__(self, database: 'FADatabase', table: str):
         self.database: 'FADatabase' = database
         self.table: str = table.upper()
-        self.columns_info_: List[Tuple[str, str]] = []
-        self.columns_: List[str] = []
-        self.column_id_: str = ""
-        self.list_columns: List[str] = list_columns.get(self.table, [])
+        self.list_columns: list[str] = list_columns.get(self.table, [])
 
     def __len__(self) -> int:
         return self.database.connection.execute(f"SELECT COUNT(*) FROM {self.table}").fetchone()[0]
@@ -90,28 +86,25 @@ class FADatabaseTable:
     def __iter__(self) -> Generator[Entry, None, None]:
         return self.cursor_to_dict(self.select())
 
-    @property
-    def columns_info(self) -> List[Tuple[str, str]]:
-        self.columns_info_ = [
+    @cached_property
+    def columns_info(self) -> list[tuple[str, str]]:
+        return [
             info[1:]
             for info in self.database.connection.execute(f"pragma table_info({self.table})")
-        ] if not self.columns_info_ else self.columns_info_
-        return self.columns_info_
+        ]
 
-    @property
-    def columns(self) -> List[str]:
-        self.columns_ = [name for name, *_ in self.columns_info] if not self.columns_ else self.columns_
-        return self.columns_
+    @cached_property
+    def columns(self) -> list[str]:
+        return [name for name, *_ in self.columns_info]
 
-    @property
+    @cached_property
     def column_id(self) -> str:
-        self.column_id_ = [
+        return [
             name
             for name, *_, pk in self.columns_info if pk == 1
         ][0]
-        return self.column_id_
 
-    def add_to_list(self, key: Key, values: Dict[str, List[Value]]) -> bool:
+    def add_to_list(self, key: Key, values: dict[str, list[Value]]) -> bool:
         if not (values := {k.upper(): v for k, v in values.items() if v and k.upper() in self.list_columns}):
             return False
         elif not (item := self[key]):
@@ -121,7 +114,7 @@ class FADatabaseTable:
         self.update({k: self.format_list(v) for k, v in item_new.items()}, key) if item_new != item else None
         return item_new != item
 
-    def remove_from_list(self, key: Key, values: Dict[str, List[Value]]) -> bool:
+    def remove_from_list(self, key: Key, values: dict[str, list[Value]]) -> bool:
         if not (values := {k.upper(): v for k, v in values.items() if v and k.upper() in self.list_columns}):
             return False
         elif not (item := self[key]):
@@ -133,28 +126,30 @@ class FADatabaseTable:
 
     def reload(self):
         self.__init__(self.database, self.table)
+        for attr in filter(self.__dict__.__contains__, ("columns", "columns_info", "column_id")):
+            del self.__dict__[attr]
 
-    def cursor_to_dict(self, cursor: Cursor, columns: List[str] = None) -> Generator[Entry, None, None]:
+    def cursor_to_dict(self, cursor: Cursor, columns: list[str] = None) -> Generator[Entry, None, None]:
         columns = list(map(str.upper, self.columns if columns is None else columns))
         return ({k: self.unpack_list(v) if k in self.list_columns else v for k, v in zip(columns, entry)}
                 for entry in cursor)
 
     @staticmethod
-    def format_list(obj: List[Value]) -> str:
+    def format_list(obj: list[Value]) -> str:
         return "".join(f"|{e}|" for e in sorted(set(map(str, obj)), key=str.lower))
 
     @staticmethod
-    def unpack_list(obj: str) -> List[str]:
+    def unpack_list(obj: str) -> list[str]:
         return [e for e in obj.split("|") if e]
 
-    def format_dict(self, obj: Entry) -> Dict[str, Value]:
+    def format_dict(self, obj: Entry) -> dict[str, Value]:
         obj = {k.upper().replace("_", ""): v for k, v in obj.items()}
         obj = {k: self.format_list(v) if isinstance(v := obj.get(k, ""), list) else v for k in self.columns}
         return obj
 
-    def select(self, query: Dict[str, Union[List[Value], Value]] = None, columns: List[str] = None,
+    def select(self, query: dict[str, Union[list[Value], Value]] = None, columns: list[str] = None,
                query_and: bool = True, query_and_values: bool = False, like: bool = False,
-               order: List[str] = None, limit: int = 0, offset: int = 0
+               order: list[str] = None, limit: int = 0, offset: int = 0
                ) -> Cursor:
         query = {} if query is None else query
         query = {k: [v] if not isinstance(v, list) else v for k, v in query.items()}
@@ -178,8 +173,8 @@ class FADatabaseTable:
             [v for values in query.values() for v in values]
         )
 
-    def select_sql(self, where: str = "", values: List[Value] = None, columns: List[str] = None,
-                   order: List[str] = None, limit: int = 0, offset: int = 0) -> Cursor:
+    def select_sql(self, where: str = "", values: list[Value] = None, columns: list[str] = None,
+                   order: list[str] = None, limit: int = 0, offset: int = 0) -> Cursor:
         columns = self.columns if columns is None else columns
         order = [] if order is None else order
         return self.database.connection.execute(
@@ -191,15 +186,15 @@ class FADatabaseTable:
             [] if values is None else values
         )
 
-    def insert(self, values: Dict[str, Value], replace: bool = True):
+    def insert(self, values: dict[str, Value], replace: bool = True):
         self.database.connection.execute(
             f"""INSERT OR {'REPLACE' if replace else 'IGNORE'} INTO {self.table}
             ({','.join(values.keys())}) VALUES ({','.join(['?'] * len(values))})""",
             [v for v in values.values()]
         )
 
-    def update(self, values: Dict[str, Value], key: Optional[Key] = None):
-        update_columns: List[str] = [f"{col} = ?" for col in values]
+    def update(self, values: dict[str, Value], key: Optional[Key] = None):
+        update_columns: list[str] = [f"{col} = ?" for col in values]
         where_str: str = f"WHERE {self.column_id} = ?" if key else ""
         self.database.connection.execute(
             f"UPDATE {self.table} SET {','.join(update_columns)} {where_str}",
@@ -211,7 +206,7 @@ class FADatabaseTable:
 
 
 class FADatabaseJournals(FADatabaseTable):
-    def save_journal(self, journal: Dict[str, Union[int, str, list]]):
+    def save_journal(self, journal: dict[str, Union[int, str, list]]):
         journal = self.format_dict(journal)
         self[journal["ID"]] = journal
 
@@ -233,7 +228,7 @@ class FADatabaseSettings(FADatabaseTable):
     def __setitem__(self, setting: str, value: str):
         self.insert({"SETTING": setting, "SVALUE": value})
 
-    def read_history(self) -> List[Tuple[float, str]]:
+    def read_history(self) -> list[tuple[float, str]]:
         return list(map(tuple, loads(h) if (h := self["HISTORY"]) else []))
 
     def add_history(self, command: str, time: float = 0):
@@ -244,8 +239,9 @@ class FADatabaseSettings(FADatabaseTable):
 
 
 class FADatabaseSubmissions(FADatabaseTable):
-    def save_submission_file(self, submission_id: int, file: bytes, name: str, ext: str, guess_ext: bool = True) -> str:
-        if not file:
+    def save_submission_file(self, submission_id: int, file: Optional[bytes], name: str, ext: str,
+                             guess_ext: bool = True) -> str:
+        if file is None:
             return ""
 
         ext = guess_extension(file, ext) if guess_ext else ext
@@ -257,12 +253,17 @@ class FADatabaseSubmissions(FADatabaseTable):
 
         return ext
 
-    def save_submission(self, submission: Dict[str, Union[int, str, list]], file: Optional[bytes] = None):
+    def save_submission_thumbnail(self, submission_id: int, file: Optional[bytes]):
+        self.save_submission_file(submission_id, file, "thumbnail", "jpg", False)
+
+    def save_submission(self, submission: dict[str, Union[int, str, list]], file: Optional[bytes] = None,
+                        thumbnail: Optional[bytes] = None):
         submission = self.format_dict(submission)
 
         submission["FILEEXT"] = name.split(".")[-1] if "." in (name := submission["FILEURL"].split("/")[-1]) else ""
         submission["FILEEXT"] = self.save_submission_file(submission["ID"], file, "submission", submission["FILEEXT"])
-        submission["FILESAVED"] = bool(file)
+        submission["FILESAVED"] = (10 * bool(file)) + bool(thumbnail)
+        self.save_submission_thumbnail(submission["ID"], thumbnail)
 
         self[submission["ID"]] = submission
 
@@ -285,10 +286,11 @@ class FADatabaseSubmissions(FADatabaseTable):
 
 
 class FADatabaseUsers(FADatabaseTable):
-    def new_user(self, user: str):
+    def new_user(self, user: str) -> str:
         user = clean_username(user)
         if user not in self:
             self[user] = {f: "" for f in self.columns}
+        return user
 
     def activate_user(self, user: str):
         if (user_entry := self[(user := clean_username(user))]) is None:
@@ -315,7 +317,7 @@ class FADatabaseUsers(FADatabaseTable):
 
 class FADatabase:
     def __init__(self, database_path: str):
-        self.database_path: str = database_path
+        self.database_path: str = abspath(database_path)
         self.connection: Connection = connect(database_path)
 
         if journals_table not in (tables := self.tables):
@@ -336,7 +338,7 @@ class FADatabase:
     def __getitem__(self, table: str):
         return FADatabaseTable(self, table)
 
-    def __iter__(self) -> Generator[Tuple[str, FADatabaseTable], None, None]:
+    def __iter__(self) -> Generator[tuple[str, FADatabaseTable], None, None]:
         return ((table, FADatabaseTable(self, table)) for table in self.tables)
 
     def __contains__(self, table: str) -> bool:
@@ -349,7 +351,7 @@ class FADatabase:
         self.close()
 
     @property
-    def tables(self) -> List[str]:
+    def tables(self) -> list[str]:
         return [
             name
             for [name] in self.connection.execute(
@@ -395,7 +397,7 @@ class FADatabase:
     def close(self):
         self.connection.close()
 
-    def update(self, db_b: 'FADatabase'):
+    def merge(self, db_b: 'FADatabase'):
         merge_database(self.connection, dirname(self.database_path), db_b.connection, dirname(db_b.database_path))
 
     def vacuum(self):
